@@ -16,11 +16,12 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-#ident "$Id: reaper.c,v 1.11 2003/05/19 19:49:11 nalin Exp $"
+#ident "$Id: reaper.c,v 1.14 2006/03/15 11:02:59 behdad Exp $"
 #include "../config.h"
 #include <sys/types.h>
 #include <sys/wait.h>
 #include <signal.h>
+#include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
 #include <glib.h>
@@ -32,16 +33,8 @@
 
 #ifdef HAVE_LOCALE_H
 #include <locale.h>
-#else
-#define bindtextdomain(package,dir)
-#endif
-
-#ifdef ENABLE_NLS
-#include <libintl.h>
-#define _(String) dgettext(PACKAGE, String)
-#else
-#define _(String) String
 #endif
+#include <glib/gi18n-lib.h>
 
 static VteReaper *singleton_reaper = NULL;
 struct reaper_info {
@@ -86,12 +79,57 @@ vte_reaper_emit_signal(GIOChannel *channel, GIOCondition condition,
 	g_assert(data == singleton_reaper);
 	read(singleton_reaper->iopipe[0], &info, sizeof(info));
 	if (info.signum == SIGCHLD) {
+		if (_vte_debug_on(VTE_DEBUG_SIGNALS)) {
+			fprintf(stderr, "Reaper emitting child-exited "
+				"signal.\n");
+		}
 		g_signal_emit_by_name(data, "child-exited",
 				      info.pid, info.status);
 	}
 	return TRUE;
 }
 
+#if GLIB_CHECK_VERSION(2,4,0)
+static void
+vte_reaper_child_watch_cb(GPid pid, gint status, gpointer data)
+{
+	if (_vte_debug_on(VTE_DEBUG_SIGNALS)) {
+		fprintf(stderr, "Reaper emitting child-exited signal.\n");
+	}
+	g_signal_emit_by_name(data, "child-exited", pid, status);
+}
+#endif
+
+static void
+vte_reaper_child_watch_destroyed(gpointer data)
+{
+	/* no-op */
+}
+
+/**
+ * vte_reaper_add_child:
+ * @pid: the ID of a child process which will be monitored
+ *
+ * Ensures that child-exited signals will be emitted when @pid exits.  This is
+ * necessary for correct operation when running with glib versions >= 2.4.
+ *
+ * Returns: the new source ID
+ *
+ * Since 0.11.11
+ */
+int
+vte_reaper_add_child(GPid pid)
+{
+#if GLIB_CHECK_VERSION(2,4,0)
+	return g_child_watch_add_full(G_PRIORITY_HIGH,
+				      pid,
+				      vte_reaper_child_watch_cb,
+				      vte_reaper_get(),
+				      vte_reaper_child_watch_destroyed);
+#endif
+	return VTE_INVALID_SOURCE;
+}
+
 static void
 vte_reaper_channel_destroyed(gpointer data)
 {
@@ -113,6 +151,13 @@ vte_reaper_init(VteReaper *reaper, gpointer *klass)
 	/* Create the channel. */
 	reaper->channel = g_io_channel_unix_new(reaper->iopipe[0]);
 
+#if GLIB_CHECK_VERSION(2,4,0)
+	if ((glib_major_version > 2) || /* 3.x and later */
+	    ((glib_major_version == 2) && (glib_minor_version >= 4))) {/* 2.4 */
+		return;
+	}
+#endif
+
 	/* Add the channel to the source list. */
 	g_io_add_watch_full(reaper->channel,
 			    G_PRIORITY_HIGH,
@@ -126,6 +171,9 @@ vte_reaper_init(VteReaper *reaper, gpointer *klass)
 	sigemptyset(&action.sa_mask);
 	action.sa_flags = SA_RESTART | SA_NOCLDSTOP;
 	sigaction(SIGCHLD, &action, NULL);
+	if (_vte_debug_on(VTE_DEBUG_SIGNALS)) {
+		fprintf(stderr, "Hooked SIGCHLD signal in reaper.\n");
+	}
 }
 
 static void
@@ -142,6 +190,9 @@ vte_reaper_finalize(GObject *reaper)
 	if (old_action.sa_handler == vte_reaper_signal_handler) {
 		sigaction(SIGCHLD, &action, NULL);
 	}
+	if (_vte_debug_on(VTE_DEBUG_SIGNALS)) {
+		fprintf(stderr, "Unhooked SIGCHLD signal in reaper.\n");
+	}
 
 	/* Remove the channel from the source list. */
 	g_source_remove_by_user_data(reaper);
