@@ -16,7 +16,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-#ident "$Id: trie.c,v 1.34 2003/05/30 00:35:36 nalin Exp $"
+#ident "$Id: trie.c,v 1.36 2006/01/30 12:23:49 gpastore Exp $"
 #include "../config.h"
 #include <sys/types.h>
 #include <assert.h>
@@ -648,9 +648,12 @@ _vte_trie_matchx(struct _vte_trie *trie, const gunichar *pattern, size_t length,
 	/* We're done searching.  Copy out any parameters we picked up. */
 	if (bestarray != NULL) {
 		for (i = 0; i < bestarray->n_values; i++) {
-			g_value_array_append(array,
-					     g_value_array_get_nth(bestarray,
-								   i));
+			GValue *value = g_value_array_get_nth(bestarray, i);
+			g_value_array_append(array, value);
+			
+			if (G_VALUE_HOLDS_POINTER(value)) {
+				g_value_set_pointer(value, NULL);
+			}
 		}
 		_vte_matcher_free_params_array(bestarray);
 	}
@@ -704,6 +707,7 @@ _vte_trie_match(struct _vte_trie *trie, const gunichar *pattern, size_t length,
 					if (ptr != NULL) {
 						g_free(ptr);
 					}
+					g_value_set_pointer(value, NULL);
 				}
 			}
 			_vte_matcher_free_params_array(valuearray);
@@ -847,7 +851,7 @@ convert_mbstowcs(const char *i, size_t ilen,
 
 	memset(o, 0, max_olen);
 	outlen = max_olen;
-	_vte_conv(conv, (char**)&i, &ilen, (char**)&o, &outlen);
+	_vte_conv_cu(conv, (char**)&i, &ilen, &o, &outlen);
 	_vte_conv_close(conv);
 
 	*olen = (max_olen - outlen) / sizeof(gunichar);
